/* eslint-disable no-undef */

import { Stream } from "stream";

/**
 * 
 * @returns {Promise<Stream>}
 */
async function getVcdStream() {
    const ostream = await window.getVcdStream();
    return ostream;
}

/**
 * @returns {Promise<Uint8Array>}
 */
async function readVcdFile() {
    const response = await fetch('cpu.vcd');
    const blob = await response.blob();
    const reader = new FileReader();
    return new Promise((resolve, reject) => {
        reader.onload = event => {
            const fileContent = event.target.result;
            const unintarray = new Uint8Array(fileContent);
            resolve(unintarray);     
        };
        reader.readAsArrayBuffer(blob);
    });
}

function debounceWrapper(fn, delay) {
    let timer
    return function () {
        if (timer) {
            clearTimeout(timer)
        }
        timer = setTimeout(() => {
            fn()
        }, delay)
    }
}

/**
 * @param { string } searchString
 * @param {{
 * kind: 'var' | 'scope',
 * type: 'wire' | 'reg' | 'module',
 * name: string,
 * size?: number,
 * link?: string
 * }} module 
 * @param { Set<string> } searchScope
 * @param { boolean } caseSensitivity
 * @param { boolean } displayParentOnly
 * @returns { { htmlString: string, module } | null }
*/
function makeSearchResultItem(searchString, module, searchScope, caseSensitivity, displayParentOnly) {
    if (searchScope.has(module.type)) {
        let pattern = module.name;
        if (!caseSensitivity) {
            pattern = pattern.toLowerCase();
            searchString = searchString.toLowerCase();
        }
        if (pattern.includes(searchString)) {
            let p = module;

            const deps = [];
            while (p) {
                if (p.name && p.type) {
                    deps.push(p);
                }
                p = p.parent;
                if (displayParentOnly && deps.length == 2) {
                    break;
                }
            }

            let htmlString = '';
            for (let i = deps.length - 1; i >= 0; -- i) {
                const mod = deps[i];
                if (mod.type === 'module') {
                    htmlString += '<span class="iconfont icon-memory-chip"></span>&ensp;' + mod.name;
                } else {
                    htmlString += '<span class="iconfont icon-wave-square"></span>&ensp;' + mod.name;
                }
                if (i > 0) {
                    htmlString += '<div class="dep-arrow"></div>';
                }
            }
            return {
                htmlString,
                module
            };
        } 
    }
    return null;
}


export {
    getVcdStream,
    readVcdFile,
    debounceWrapper,
    makeSearchResultItem
};